@@ -2,6 +2,8 @@
 see templateexporter.py.
 """
 
+# Copyright (c) Jupyter Development Team.
+# Distributed under the terms of the Modified BSD License.
 
 from __future__ import print_function, absolute_import
 
@@ -15,7 +17,7 @@ from traitlets.config.configurable import LoggingConfigurable
 from traitlets.config import Config
 import nbformat
 from traitlets import HasTraits, Unicode, List, TraitError
-from ipython_genutils.importstring import import_item
+from traitlets.utils.importstring import import_item
 from ipython_genutils import text, py3compat
 
 
@@ -49,10 +51,9 @@ class Exporter(LoggingConfigurable):
     accompanying resources dict.
     """
 
-    file_extension = FilenameExtension(
-        '.txt', config=True,
+    file_extension = FilenameExtension('.txt',
         help="Extension of the file that should be written to disk"
-        )
+    ).tag(config=True)
 
     # MIME type of the result file, for HTTP response headers.
     # This is *not* a traitlet, because we want to be able to access it from
@@ -60,8 +61,9 @@ class Exporter(LoggingConfigurable):
     output_mimetype = ''
 
     #Configurability, allows the user to easily add filters and preprocessors.
-    preprocessors = List(config=True,
-        help="""List of preprocessors, by name or namespace, to enable.""")
+    preprocessors = List(
+        help="""List of preprocessors, by name or namespace, to enable."""
+    ).tag(config=True)
 
     _preprocessors = List()
 
@@ -75,10 +77,9 @@ class Exporter(LoggingConfigurable):
                                   'nbconvert.preprocessors.HighlightMagicsPreprocessor',
                                   'nbconvert.preprocessors.ExtractOutputPreprocessor',
                               ],
-        config=True,
         help="""List of preprocessors available by default, by name, namespace, 
-        instance, or type.""")
-
+        instance, or type."""
+    ).tag(config=True)
 
     def __init__(self, config=None, **kw):
         """
@@ -200,7 +201,7 @@ class Exporter(LoggingConfigurable):
 
         """
         if preprocessor is None:
-            raise TypeError('preprocessor')
+            raise TypeError('preprocessor must not be None')
         isclass = isinstance(preprocessor, type)
         constructed = not isclass
 
@@ -231,7 +232,7 @@ class Exporter(LoggingConfigurable):
         else:
             # Preprocessor is an instance of something without a __call__ 
             # attribute.  
-            raise TypeError('preprocessor')
+            raise TypeError('preprocessor must be callable or an importable constructor, got %r' % preprocessor)
 
 
     def _init_preprocessors(self):
