@@ -1,4 +1,5 @@
-"""Module containing a preprocessor that removes the outputs from code cells"""
+"""Module containing a preprocessor that executes the code cells
+and updates outputs"""
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
@@ -11,12 +12,13 @@ try:
 except ImportError:
     from Queue import Empty  # Py 2
 
-from traitlets import List, Unicode, Bool
+from traitlets import List, Unicode, Bool, Enum, Any, Type
 
 from nbformat.v4 import output_from_msg
 from .base import Preprocessor
 from ..utils.exceptions import ConversionException
 from traitlets import Integer
+from jupyter_client.manager import KernelManager
 
 
 class CellExecutionError(ConversionException):
@@ -30,6 +32,12 @@ class CellExecutionError(ConversionException):
         self.traceback = traceback
 
     def __str__(self):
+        s = self.__unicode__()
+        if not isinstance(s, str):
+            s = s.encode('utf8', 'replace')
+        return s
+
+    def __unicode__(self):
         return self.traceback
 
 
@@ -38,20 +46,39 @@ class ExecutePreprocessor(Preprocessor):
     Executes all the cells in a notebook
     """
 
-    timeout = Integer(30, config=True, allow_none=True,
+    timeout = Integer(30, allow_none=True,
         help=dedent(
             """
             The time to wait (in seconds) for output from executions.
             If a cell execution takes longer, an exception (TimeoutError
             on python 3+, RuntimeError on python 2) is raised.
 
-            `None` or `-1` will disable the timeout.
+            `None` or `-1` will disable the timeout. If `timeout_func` is set,
+            it overrides `timeout`.
             """
         )
-    )
+    ).tag(config=True)
 
-    interrupt_on_timeout = Bool(
-        False, config=True,
+    timeout_func = Any(
+        default_value=None,
+        allow_none=True,
+        help=dedent(
+            """
+            A callable which, when given the cell source as input,
+            returns the time to wait (in seconds) for output from cell
+            executions. If a cell execution takes longer, an exception
+            (TimeoutError on python 3+, RuntimeError on python 2) is
+            raised.
+
+            Returning `None` or `-1` will disable the timeout for the cell.
+            Not setting `timeout_func` will cause the preprocessor to
+            default to using the `timeout` trait for all cells. The
+            `timeout_func` trait overrides `timeout` if it is not `None`.
+            """
+        )
+    ).tag(config=True)
+
+    interrupt_on_timeout = Bool(False,
         help=dedent(
             """
             If execution of a cell times out, interrupt the kernel and
@@ -59,36 +86,33 @@ class ExecutePreprocessor(Preprocessor):
             stopping.
             """
         )
-    )
+    ).tag(config=True)
 
-    allow_errors = Bool(
-        False, config=True,
+    allow_errors = Bool(False,
         help=dedent(
             """
             If `False` (default), when a cell raises an error the
-            execution is stoppped and a `CellExecutionError`
+            execution is stopped and a `CellExecutionError`
             is raised.
             If `True`, execution errors are ignored and the execution
             is continued until the end of the notebook. Output from
             exceptions is included in the cell output in both cases.
             """
         )
-    )
+    ).tag(config=True)
 
     extra_arguments = List(Unicode())
 
-    kernel_name = Unicode(
-        '', config=True,
+    kernel_name = Unicode('',
         help=dedent(
             """
             Name of kernel to use to execute the cells.
             If not set, use the kernel_spec embedded in the notebook.
             """
         )
-    )
+    ).tag(config=True)
 
-    raise_on_iopub_timeout = Bool(
-        False, config=True,
+    raise_on_iopub_timeout = Bool(False,
         help=dedent(
             """
             If `False` (default), then the kernel will continue waiting for
@@ -100,8 +124,25 @@ class ExecutePreprocessor(Preprocessor):
             notebooks with memory-consuming infinite loops.
             """
             )
-        )
+    ).tag(config=True)
 
+    shutdown_kernel = Enum(['graceful', 'immediate'],
+        default_value='graceful',
+        help=dedent(
+            """
+            If `graceful` (default), then the kernel is given time to clean
+            up after executing all cells, e.g., to execute its `atexit` hooks.
+            If `immediate`, then the kernel is signaled to immediately
+            terminate.
+            """
+            )
+    ).tag(config=True)
+
+    kernel_manager_class = Type(
+        default_value=KernelManager,
+        config=True,
+        help='The kernel manager class to use.'
+    )
 
     def preprocess(self, nb, resources):
         """
@@ -129,7 +170,23 @@ class ExecutePreprocessor(Preprocessor):
         if path == '':
             path = None
 
-        from jupyter_client.manager import start_new_kernel
+        # from jupyter_client.manager import start_new_kernel
+
+        def start_new_kernel(startup_timeout=60, kernel_name='python',
+                             **kwargs):
+            km = self.kernel_manager_class(kernel_name=kernel_name)
+            km.start_kernel(**kwargs)
+            kc = km.client()
+            kc.start_channels()
+            try:
+                kc.wait_for_ready(timeout=startup_timeout)
+            except RuntimeError:
+                kc.stop_channels()
+                km.shutdown_kernel()
+                raise
+
+            return km, kc
+
         kernel_name = nb.metadata.get('kernelspec', {}).get('name', 'python')
         if self.kernel_name:
             kernel_name = self.kernel_name
@@ -137,7 +194,6 @@ class ExecutePreprocessor(Preprocessor):
         self.km, self.kc = start_new_kernel(
             kernel_name=kernel_name,
             extra_arguments=self.extra_arguments,
-            stderr=open(os.devnull, 'w'),
             cwd=path)
         self.kc.allow_stdin = False
 
@@ -145,7 +201,7 @@ class ExecutePreprocessor(Preprocessor):
             nb, resources = super(ExecutePreprocessor, self).preprocess(nb, resources)
         finally:
             self.kc.stop_channels()
-            self.km.shutdown_kernel(now=True)
+            self.km.shutdown_kernel(now=self.shutdown_kernel == 'immediate')
 
         return nb, resources
 
@@ -164,7 +220,7 @@ class ExecutePreprocessor(Preprocessor):
         if not self.allow_errors:
             for out in outputs:
                 if out.output_type == 'error':
-                    pattern = """\
+                    pattern = u"""\
                         An error occurred while executing the following cell:
                         ------------------
                         {cell.source}
@@ -183,18 +239,17 @@ class ExecutePreprocessor(Preprocessor):
         # wait for finish, with timeout
         while True:
             try:
-                timeout = self.timeout
-                if timeout < 0:
+                if self.timeout_func is not None:
+                    timeout = self.timeout_func(cell)
+                else:
+                    timeout = self.timeout
+
+                if not timeout or timeout < 0:
                     timeout = None
                 msg = self.kc.shell_channel.get_msg(timeout=timeout)
             except Empty:
-                self.log.error("""Timeout waiting for execute reply (%is).
-                If your cell should take longer than this, you can increase the timeout with:
-
-                    c.ExecutePreprocessor.timeout = SECONDS
-
-                in jupyter_nbconvert_config.py
-                """ % self.timeout)
+                self.log.error(
+                    "Timeout waiting for execute reply (%is)." % self.timeout)
                 if self.interrupt_on_timeout:
                     self.log.error("Interrupting kernel")
                     self.km.interrupt_kernel()
@@ -204,8 +259,7 @@ class ExecutePreprocessor(Preprocessor):
                         exception = TimeoutError
                     except NameError:
                         exception = RuntimeError
-                    raise exception("Cell execution timed out, see log"
-                                    " for details.")
+                    raise exception("Cell execution timed out")
 
             if msg['parent_header'].get('msg_id') == msg_id:
                 break
